@@ -1,4 +1,4 @@
-/*	$OpenBSD: client.c,v 1.81 2008/06/10 03:51:53 naddy Exp $ */
+/*	$OpenBSD: client.c,v 1.88 2009/06/24 17:34:32 henning Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -20,6 +20,7 @@
 #include <sys/param.h>
 #include <errno.h>
 #include <md5.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -122,7 +123,7 @@ client_nextaddr(struct ntp_peer *p)
 int
 client_query(struct ntp_peer *p)
 {
-	int	tos = IPTOS_LOWDELAY;
+	int	val;
 
 	if (p->addr == NULL && client_nextaddr(p) == -1) {
 		set_next(p, MAX(SETTIME_TIMEOUT,
@@ -149,9 +150,14 @@ client_query(struct ntp_peer *p)
 			} else
 				fatal("client_query connect");
 		}
+		val = IPTOS_LOWDELAY;
 		if (p->addr->ss.ss_family == AF_INET && setsockopt(p->query->fd,
-		    IPPROTO_IP, IP_TOS, &tos, sizeof(tos)) == -1)
+		    IPPROTO_IP, IP_TOS, &val, sizeof(val)) == -1)
 			log_warn("setsockopt IPTOS_LOWDELAY");
+		val = 1;
+		if (setsockopt(p->query->fd, SOL_SOCKET, SO_TIMESTAMP,
+		    &val, sizeof(val)) == -1)
+			fatal("setsockopt SO_TIMESTAMP");
 	}
 
 	/*
@@ -190,25 +196,68 @@ client_query(struct ntp_peer *p)
 int
 client_dispatch(struct ntp_peer *p, u_int8_t settime)
 {
-	char			 buf[NTP_MSGSIZE];
-	ssize_t			 size;
 	struct ntp_msg		 msg;
+	struct msghdr		 somsg;
+	struct iovec		 iov[1];
+	struct timeval		 tv;
+	char			 buf[NTP_MSGSIZE];
+	union {
+		struct cmsghdr	hdr;
+		char		buf[CMSG_SPACE(sizeof(tv))];
+	} cmsgbuf;
+	struct cmsghdr		*cmsg;
+	ssize_t			 size;
 	double			 T1, T2, T3, T4;
 	time_t			 interval;
 
-	if ((size = recvfrom(p->query->fd, &buf, sizeof(buf), 0,
-	    NULL, NULL)) == -1) {
+	bzero(&somsg, sizeof(somsg));
+	iov[0].iov_base = buf;
+	iov[0].iov_len = sizeof(buf);
+	somsg.msg_iov = iov;
+	somsg.msg_iovlen = 1;
+	somsg.msg_control = cmsgbuf.buf;
+	somsg.msg_controllen = sizeof(cmsgbuf.buf);
+
+	T4 = getoffset();
+	if ((size = recvmsg(p->query->fd, &somsg, 0)) == -1) {
 		if (errno == EHOSTUNREACH || errno == EHOSTDOWN ||
 		    errno == ENETUNREACH || errno == ENETDOWN ||
-		    errno == ECONNREFUSED || errno == EADDRNOTAVAIL) {
-			client_log_error(p, "recvfrom", errno);
+		    errno == ECONNREFUSED || errno == EADDRNOTAVAIL ||
+		    errno == ENOPROTOOPT || errno == ENOENT) {
+			client_log_error(p, "recvmsg", errno);
 			set_next(p, error_interval());
 			return (0);
 		} else
 			fatal("recvfrom");
 	}
 
-	T4 = gettime_corrected();
+	if (somsg.msg_flags & MSG_TRUNC) {
+		client_log_error(p, "recvmsg packet", EMSGSIZE);
+		set_next(p, error_interval());
+		return (0);
+	}
+
+	if (somsg.msg_flags & MSG_CTRUNC) {
+		client_log_error(p, "recvmsg control data", E2BIG);
+		set_next(p, error_interval());
+		return (0);
+	}
+
+	for (cmsg = CMSG_FIRSTHDR(&somsg); cmsg != NULL;
+	    cmsg = CMSG_NXTHDR(&somsg, cmsg)) {
+		if (cmsg->cmsg_level == SOL_SOCKET &&
+		    cmsg->cmsg_type == SCM_TIMESTAMP) {
+			memcpy(&tv, CMSG_DATA(cmsg), sizeof(tv));
+			T4 += tv.tv_sec + JAN_1970 + 1.0e-6 * tv.tv_usec;
+			break;
+		}
+	}
+
+	if (T4 < JAN_1970) {
+		client_log_error(p, "recvmsg control format", EBADF);
+		set_next(p, error_interval());
+		return (0);
+	}
 
 	ntp_getmsg((struct sockaddr *)&p->addr->ss, buf, size, &msg);
 
@@ -218,10 +267,21 @@ client_dispatch(struct ntp_peer *p, u_int8_t settime)
 
 	if ((msg.status & LI_ALARM) == LI_ALARM || msg.stratum == 0 ||
 	    msg.stratum > NTP_MAXSTRATUM) {
+		char s[16];
+
+		if ((msg.status & LI_ALARM) == LI_ALARM) {
+			strlcpy(s, "alarm", sizeof(s));
+		} else if (msg.stratum == 0) {
+			/* Kiss-o'-Death (KoD) packet */
+			strlcpy(s, "KoD", sizeof(s));
+		} else if (msg.stratum > NTP_MAXSTRATUM) {
+			snprintf(s, sizeof(s), "stratum %d", msg.stratum);
+		}
 		interval = error_interval();
 		set_next(p, interval);
-		log_info("reply from %s: not synced, next query %ds",
-		    log_sockaddr((struct sockaddr *)&p->addr->ss), interval);
+		log_info("reply from %s: not synced (%s), next query %ds",
+		    log_sockaddr((struct sockaddr *)&p->addr->ss), s,
+			interval);
 		return (0);
 	}
 
@@ -244,6 +304,16 @@ client_dispatch(struct ntp_peer *p, u_int8_t settime)
 	T2 = lfp_to_d(msg.rectime);
 	T3 = lfp_to_d(msg.xmttime);
 
+	/*
+	 * XXX workaround: time_t / tv_sec must never wrap.
+	 * around 2020 we will need a solution (64bit time_t / tv_sec).
+	 * consider every answer with a timestamp beyond january 2030 bogus.
+	 */
+	if (T2 > JAN_2030 || T3 > JAN_2030) {
+		set_next(p, error_interval());
+		return (0);
+	}
+
 	p->reply[p->shift].offset = ((T2 - T1) + (T3 - T4)) / 2;
 	p->reply[p->shift].delay = (T4 - T1) - (T3 - T2);
 	if (p->reply[p->shift].delay < 0) {
@@ -263,7 +333,7 @@ client_dispatch(struct ntp_peer *p, u_int8_t settime)
 	p->reply[p->shift].status.precision = msg.precision;
 	p->reply[p->shift].status.rootdelay = sfp_to_d(msg.rootdelay);
 	p->reply[p->shift].status.rootdispersion = sfp_to_d(msg.dispersion);
-	p->reply[p->shift].status.refid = ntohl(msg.refid);
+	p->reply[p->shift].status.refid = msg.refid;
 	p->reply[p->shift].status.reftime = lfp_to_d(msg.reftime);
 	p->reply[p->shift].status.poll = msg.ppoll;
 	p->reply[p->shift].status.stratum = msg.stratum;
