@@ -1,4 +1,4 @@
-/*	$OpenBSD: ntpd.c,v 1.61 2008/07/19 21:31:39 claudio Exp $ */
+/*	$OpenBSD: ntpd.c,v 1.66 2009/06/06 18:14:25 pyr Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -144,8 +144,6 @@ main(int argc, char *argv[])
 	if ((pw = getpwnam(NTPD_USER)) == NULL)
 		errx(1, "unknown user %s", NTPD_USER);
 
-	endpwent();
-
 	reset_adjtime();
 	if (!lconf.settime) {
 		log_init(lconf.debug);
@@ -291,7 +289,8 @@ dispatch_imsg(struct ntpd_conf *lconf)
 				fatalx("invalid IMSG_ADJTIME received");
 			memcpy(&d, imsg.data, sizeof(d));
 			n = ntpd_adjtime(d);
-			imsg_compose(ibuf, IMSG_ADJTIME, 0, 0, &n, sizeof(n));
+			imsg_compose(ibuf, IMSG_ADJTIME, 0, 0, -1,
+			     &n, sizeof(n));
 			break;
 		case IMSG_ADJFREQ:
 			if (imsg.hdr.len != IMSG_HEADER_SIZE + sizeof(d))
@@ -362,8 +361,8 @@ ntpd_adjtime(double d)
 	static int	firstadj = 1;
 
 	d += getoffset();
-	if (d >= (double)LOG_NEGLIGEE / 1000 ||
-	    d <= -1 * (double)LOG_NEGLIGEE / 1000)
+	if (d >= (double)LOG_NEGLIGIBLE_ADJTIME / 1000 ||
+	    d <= -1 * (double)LOG_NEGLIGIBLE_ADJTIME / 1000)
 		log_info("adjusting local clock by %fs", d);
 	else
 		log_debug("adjusting local clock by %fs", d);
@@ -380,6 +379,7 @@ void
 ntpd_adjfreq(double relfreq, int wrlog)
 {
 	int64_t curfreq;
+	double ppmfreq;
 	int r;
 
 	if (adjfreq(NULL, &curfreq) == -1) {
@@ -393,10 +393,18 @@ ntpd_adjfreq(double relfreq, int wrlog)
 	 */
 	curfreq += relfreq * 1e9 * (1LL << 32);
 	r = writefreq(curfreq / 1e9 / (1LL << 32));
-	if (wrlog)
-		log_info("adjusting clock frequency by %f to %fppm%s",
-		    relfreq * 1e6, curfreq / 1e3 / (1LL << 32),
-		    r ? "" : " (no drift file)");
+	ppmfreq = relfreq * 1e6;
+	if (wrlog) {
+		if (ppmfreq >= LOG_NEGLIGIBLE_ADJFREQ ||
+		    ppmfreq <= -LOG_NEGLIGIBLE_ADJFREQ)
+			log_info("adjusting clock frequency by %f to %fppm%s",
+			    ppmfreq, curfreq / 1e3 / (1LL << 32),
+			    r ? "" : " (no drift file)");
+		else
+			log_debug("adjusting clock frequency by %f to %fppm%s",
+			    ppmfreq, curfreq / 1e3 / (1LL << 32),
+			    r ? "" : " (no drift file)");
+	}
 
 	if (adjfreq(&curfreq, NULL) == -1)
 		log_warn("adjfreq failed");
@@ -450,6 +458,8 @@ readfreq(void)
 	else if (current == 0) {
 		if (fscanf(fp, "%le", &d) == 1)
 			ntpd_adjfreq(d, 0);
+		else
+			log_warnx("can't read %s", DRIFTFILE);
 	}
 	fclose(fp);
 }
